﻿'A reference to the Phidget22.NET.dll must be added to new projects in VB.Net 
'From the "Project" menu select "Add Reference" 
'From the "Add Reference" window select the "Browse" tab 
'Browse to the directory you installed the Phidgets library to 
'The default is "C:\Program Files\Phidgets" 
'Select "Phidget22.NET.dll" and click "OK" 
'To view objects that have been added, select the "View" menu 
'Select "Object Browser" 
'Then expand the Phidget22.NET.dll item 



Imports Phidget22
Imports Phidget22.Events
Imports System.IO
Imports System.Drawing
Imports System.Linq
Imports System.Reflection


Public Class Form1
    Dim WithEvents device As Phidget22.Spatial
    Dim open As ExampleUtils.ExampleUtils.CommandLineOpen
    Dim errorBox As ErrorEventBox

    Dim spatialImage As Bitmap
    Dim _imageStream As Stream

    Dim lastTimestamp() As Double = {0, 0, 0}
    Dim lastTimestampGood() As Boolean = {False, False, False}
    Dim gyroHeading() As Double = {0, 0, 0} 'degrees

    Dim compassbearingfilter As List(Of Double()) = New List(Of Double())()
    Dim compassBearingFilterSize As Integer = 10
    Dim compassBearing As Double = 0

    Dim ambientMagneticField As Double = 0.57142 'Calgary
    Const ambientGravity As Double = 1 'in g-force

    Public Sub New()
        open = New ExampleUtils.ExampleUtils.CommandLineOpen(Me)
        ' This call is required by the Windows Form Designer.
        Phidget22.Phidget.InvokeEventCallbacks = True
        InitializeComponent()
        ' Add any initialization after the InitializeComponent() call.
    End Sub

    Public Sub New(ByVal commandLine As String())
        open = New ExampleUtils.ExampleUtils.CommandLineOpen(Me)
        open.commandLine = commandLine
        Phidget22.Phidget.InvokeEventCallbacks = True
        InitializeComponent()
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        errorBox = New ErrorEventBox(Me)
        device = New Phidget22.Spatial()

        initAccelGraph()
        initGyroGraph()
        initMagFieldGraph()
        initCompassBearingGraph()

        device = New Phidget22.Spatial()

        Dim phidgetParameters As ExampleUtils.commandLineData = open.parseCmdLine()

        Try 'set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.vb, you can check there to see them
            device.Channel = phidgetParameters.Channel 'selects the channel on the device to open
            device.DeviceSerialNumber = phidgetParameters.SerialNumber 'selects the device or hub to open
            device.HubPort = phidgetParameters.HubPort 'selects th eport on the hub to open
            device.IsHubPortDevice = phidgetParameters.isHubPortDevice 'is the device a port on a vint hub?

            If phidgetParameters.isRemote Then 'are we trying to open a remote device?
                device.IsRemote = True
                Net.EnableServerDiscovery(ServerType.Device) 'turn on network scan
                If phidgetParameters.Password <> vbNullString And phidgetParameters.ServerName <> vbNullString Then
                    Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password)
                End If
            Else
                device.IsLocal = True
            End If

            device.Open() 'open the device specified by the above parameters
        Catch ex As PhidgetException
            errorBox.addMessage("Error opening the device: " + ex.Message)
        End Try
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If device.Attached Then
            device.Close()
        End If
    End Sub

    Private Sub device_Attach(ByVal sender As Object, ByVal e As AttachEventArgs) Handles device.Attach
        Dim attachedDevice As Phidget22.Spatial = CType(sender, Phidget22.Spatial)
        PhidgetInfoBox1.FillPhidgetInfo(attachedDevice)

        Dim _assembly As Assembly = Assembly.GetExecutingAssembly()
        Dim imageName As String = _assembly.GetManifestResourceNames().Where(Function(img) img.EndsWith("1056_0_Top.png")).FirstOrDefault() 'uses the image for the 1056.  works for all models
        _imageStream = _assembly.GetManifestResourceStream(imageName)
        spatialImage = New Bitmap(_imageStream)
        Dim newWidth As Integer = CType(CType(spatialImage.Width, Double) * CType(compassView.Height, Double) / CType(spatialImage.Height, Double) * 0.6, Integer)
        Dim newHeight As Integer = CType(CType(compassView.Height, Double) * 0.6, Integer)
        spatialImage = New Bitmap(spatialImage, newWidth, newWidth)


        accelBox.Visible = True
        gyroBox.Visible = True
        compassBox.Visible = True
        accelGraphBox.Visible = True
        gyroGraphBox.Visible = True
        magFieldGraphBox.Visible = True
        compassGraphBox.Visible = True
        settingsBox.Visible = True

        attachedDevice.DataInterval = 32 'setting the datarate faster than this is possible but the GUI bogs down so it isn't recommended.

        dataIntervalTrk.Unit = "ms"
        dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString()
        dataIntervalTrk.Minimum = attachedDevice.MinDataInterval
        dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString()
        dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval
        dataIntervalTrk.trkText = attachedDevice.DataInterval.ToString()

        lastTimestampGood(0) = False
        lastTimestampGood(1) = False
        lastTimestampGood(2) = False
        gyroHeading(0) = 0
        gyroHeading(1) = 0
        gyroHeading(2) = 0
    End Sub

    Private Sub device_Detach(ByVal sender As Object, ByVal e As DetachEventArgs) Handles device.Detach
        PhidgetInfoBox1.Clear()
        accelBox.Visible = False
        gyroBox.Visible = False
        compassBox.Visible = False
        accelGraphBox.Visible = False
        gyroGraphBox.Visible = False
        magFieldGraphBox.Visible = False
        compassGraphBox.Visible = False
        settingsBox.Visible = False
    End Sub

    Private Sub device_Error(ByVal sender As Object, ByVal e As Phidget22.Events.ErrorEventArgs) Handles device.Error
        errorBox.addMessage(e.Description)
    End Sub

    Private Sub device_SpatialData(ByVal sender As Object, ByVal e As SpatialSpatialDataEventArgs) Handles device.SpatialData
        Try
            If e.MagneticField.Length > 0 Then
                accelXTxt.Text = e.Acceleration(0).ToString() + " g"
                accelYTxt.Text = e.Acceleration(1).ToString() + " g"
                accelZTxt.Text = e.Acceleration(2).ToString() + " g"

                gyroXTxt.Text = e.AngularRate(0).ToString() + " °/s"
                gyroYTxt.Text = e.AngularRate(1).ToString() + " °/s"
                gyroZTxt.Text = e.AngularRate(2).ToString() + " °/s"

                compassXTxt.Text = e.MagneticField(0).ToString() + " G"
                compassYTxt.Text = e.MagneticField(1).ToString() + " G"
                compassZTxt.Text = e.MagneticField(2).ToString() + " G"

                calculateGyroHeading(e, 0) 'x axis
                calculateGyroHeading(e, 1) 'y axis
                calculateGyroHeading(e, 2) 'z axis

                calculateCompassBearing(e)
                drawAccelGraph(e.Acceleration)
                drawCompassBearingGraph()
                drawGyroGraph(e.AngularRate)

                drawMagFieldGraph(e.MagneticField)
            End If
        Catch ex As PhidgetException
            errorBox.addMessage(ex.ToString())
        End Try
    End Sub

    'this finds a magnetic north bearing, correcting for board tilt and roll as measured by the accelerometer
    'this doesn't account for dynamic acceleration - ie accelerations other than gravity will throw off the calculation
    'calculations based on AN4248
    Dim lastAngles() As Double = {0, 0, 0}
    Sub calculateCompassBearing(ByVal e As SpatialSpatialDataEventArgs)
        Dim gravity() As Double = {e.Acceleration(0), e.Acceleration(1), e.Acceleration(2)}
        Dim magField() As Double = {e.MagneticField(0), e.MagneticField(1), e.MagneticField(2)}

        'roll angle - about axis 0
        'tan(roll angle) = gy/gz
        'use atan2 so we have an output of (-180 - 180) degrees
        Dim rollAngle As Double = Math.Atan2(gravity(1), gravity(2))

        'pitch angle - about axis 1
        'tan(pitch angle) = -gx/(gy*sin(roll angle) * gz * cos(roll angle))
        'pitch angle range is (-90 - 90) degrees
        Dim pitchAngle As Double = Math.Atan(-gravity(0) / (gravity(1) * Math.Sin(rollAngle) + gravity(2) * Math.Cos(rollAngle)))

        'yaw angle - about axis 2
        'tan(yaw angle) = (mz *sin(rollangle) * gz * cos(roll angle))
        '                 (mx * cos(pitch) + my* sin(pitch) * sin(roll) + mz * sin(pitch) * cos(roll))
        'use atan2 to get our range in (-180 - 180)
        'yaw angle = 0 degrees when axis - is pointing at magnetic north
        Dim yawAngle As Double = Math.Atan2(magField(2) * Math.Sin(rollAngle) - magField(1) * Math.Cos(rollAngle),
               magField(0) * Math.Cos(pitchAngle) + magField(1) * Math.Sin(pitchAngle) * Math.Sin(rollAngle) + magField(2) * Math.Sin(pitchAngle) * Math.Cos(rollAngle))

        Dim angles() As Double = {rollAngle, pitchAngle, yawAngle}

        'we low-pass filter the angle data so that it looks nicer on-screen
        Try
            'make sure the filter buffer doesn't have values passing the -180<->180 mark
            'only roll and yaw - pitch will never have a sudden switch like that
            For i As Integer = 0 To 2
                If Math.Abs(angles(i) - lastAngles(i)) > 3 Then
                    For Each stuff As Double() In compassbearingfilter
                        If angles(i) > lastAngles(i) Then
                            stuff(i) += 360 * Math.PI / 180
                        Else
                            stuff(i) -= 360 * Math.PI / 180
                        End If
                    Next
                End If
            Next

            lastAngles = CType(angles.Clone(), Double())

            compassbearingfilter.Add(CType(angles.Clone(), Double()))
            If compassbearingfilter.Count > compassBearingFilterSize Then
                compassbearingfilter.RemoveAt(0)
            End If

            yawAngle = 0
            pitchAngle = 0
            rollAngle = 0

            For Each stuff As Double() In compassbearingfilter
                rollAngle += stuff(0)
                pitchAngle += stuff(1)
                yawAngle += stuff(2)
            Next

            yawAngle /= compassbearingfilter.Count
            pitchAngle /= compassbearingfilter.Count
            rollAngle /= compassbearingfilter.Count

            'convert radians to degrees for display
            compassBearing = yawAngle * (180 / Math.PI)

            bearingTxt.Text = compassBearing.ToString("F1") + "°"
            pitchAngleTxt.Text = (pitchAngle * (180 / Math.PI)).ToString("F1") + "°"
            rollAngleTxt.Text = (rollAngle * (180 / Math.PI)).ToString("F1") + "°"
        Catch ex As Exception
        End Try
    End Sub

    'this integrates gyro angular rate into heading over time
    Private Sub calculateGyroHeading(data As SpatialSpatialDataEventArgs, index As Integer)
        Dim gyro As Double = 0
        gyro = data.AngularRate(index)

        If lastTimestampGood(index) Then
            'calculate heading  
            Dim timechange As Double = data.Timestamp - lastTimestamp(index) 'in ms
            Dim timeChangeSeconds = CType(timechange / 1000.0, Double)

            gyroHeading(index) += timeChangeSeconds * gyro
        End If

        lastTimestamp(index) = data.Timestamp
        lastTimestampGood(index) = True
    End Sub

    Private Sub dataIntervalTrk_Scroll(sender As Object, e As EventArgs) Handles dataIntervalTrk.LogScroll
        If (MouseButtons = MouseButtons.Left) Then
            Try
                device.DataInterval = CType(dataIntervalTrk.Value, Integer)
            Catch ex As Exception
                errorBox.addMessage("Error setting data interval: " + ex.Message)
            End Try
        End If
    End Sub

    Private Sub dataIntervalTrk_LogMouseUp(sender As Object, e As EventArgs) Handles dataIntervalTrk.LogMouseUp
        Try
            device.DataInterval = CType(dataIntervalTrk.Value, Integer)
        Catch ex As Exception
            errorBox.addMessage("Error setting data interval: " + ex.Message)
        End Try
    End Sub

#Region "Graphs"
    'gyroscope graph
    Dim gyroGraphicsContext As BufferedGraphicsContext
    Dim gyroGraphicsBuffer As BufferedGraphics
    Dim gyroDrawingSurface As Bitmap

    Dim gyrocircleRectangle(3) As Rectangle
    Dim gyroboundsRectangle As Rectangle
    Dim gyroxyAxisPen(3) As Pen
    Dim gyrocirclePen As Pen
    Dim gyrocircleDiameter(3) As Single
    Dim gyrocircleRadius(3) As Single

    Private Sub initGyroGraph()
        'initialize our graphical representation of the gyroscope data
        gyroDrawingSurface = New Bitmap(gyroView.ClientRectangle.Width, gyroView.ClientRectangle.Height, Drawing.Imaging.PixelFormat.Format24bppRgb)

        gyroboundsRectangle = New Rectangle(0, 0, gyroDrawingSurface.Width, gyroDrawingSurface.Height)
        gyroboundsRectangle.Inflate(-1, -1)

        Dim xCenter As Single = CType(gyroboundsRectangle.Width / 2.0, Single)
        Dim yCenter As Single = CType(gyroboundsRectangle.Height / 2.0, Single)

        gyroxyAxisPen(0) = New Pen(Color.Red, 4)
        gyroxyAxisPen(1) = New Pen(Color.Green, 4)
        gyroxyAxisPen(2) = New Pen(Color.Blue, 4)

        gyrocirclePen = New Pen(Color.Black, 1)

        gyrocircleDiameter(0) = CType(gyroDrawingSurface.Width / 1.5, Single)
        gyrocircleDiameter(1) = CType(gyroDrawingSurface.Width / 1.6, Single)
        gyrocircleDiameter(2) = CType(gyroDrawingSurface.Width / 1.714, Single)

        For i As Integer = 0 To 2
            gyrocircleRadius(i) = CType(gyrocircleDiameter(i) / 2.0, Single)
            gyrocircleRectangle(i) = New Rectangle(CType(Math.Round(xCenter - gyrocircleRadius(i)), Integer), CType(Math.Round(yCenter - gyrocircleRadius(i)), Integer),
                                                   CType(Math.Round(gyrocircleDiameter(i)), Integer), CType(Math.Round(gyrocircleDiameter(i)), Integer))
        Next
    End Sub

    Private Sub drawGyroGraph(gyrodata As Double())
        'get our graphics buffer going so we can draw to our panel
        gyroGraphicsContext = New BufferedGraphicsContext()
        gyroGraphicsBuffer = gyroGraphicsContext.Allocate(Graphics.FromImage(gyroDrawingSurface), gyroboundsRectangle)
        gyroGraphicsBuffer.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias
        gyroGraphicsBuffer.Graphics.Clear(SystemColors.Control)


        Dim xCenter As Single = CType((gyroboundsRectangle.Width / 2.0), Single)
        Dim yCenter As Single = CType((gyroboundsRectangle.Height / 2.0), Single)

        'heading circles
        For i As Integer = 0 To 2
            gyroGraphicsBuffer.Graphics.DrawEllipse(gyrocirclePen, gyrocircleRectangle(i))
            gyroGraphicsBuffer.Graphics.DrawEllipse(
                    gyroxyAxisPen(i),
                    CType((gyrocircleRadius(i) * Math.Cos(gyroHeading(i) * (Math.PI / 180.0)) + xCenter) - 2, Single),
                    CType((-gyrocircleRadius(i) * Math.Sin(gyroHeading(i) * (Math.PI / 180.0)) + yCenter) - 2, Single),
                    4,
                    4
                )
        Next

        gyroGraphicsBuffer.Render(gyroView.CreateGraphics())

        gyroGraphicsBuffer.Dispose()

        gyroGraphicsContext.Dispose()
    End Sub

    'accelerometer graph
    Dim accelGraphicsContext As BufferedGraphicsContext
    Dim accelGraphicsBuffer As BufferedGraphics
    Dim accelDrawingSurface As Bitmap

    Dim accelcircleRectangle As Rectangle
    Dim accelboundsRectangle As Rectangle
    Dim accelxyAxisPen As Pen
    Dim accelcirclePen As Pen
    Dim accelcircleDiameter As Single
    Dim accelcircleRadius As Single

    Private Sub initAccelGraph()
        'initialize our graphical representation of the acceleration data
        accelDrawingSurface = New Bitmap(panel1.ClientRectangle.Width, panel1.ClientRectangle.Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb)

        accelboundsRectangle = New Rectangle(0, 0, accelDrawingSurface.Width, accelDrawingSurface.Height)
        accelboundsRectangle.Inflate(-1, -1)

        Dim xCenter As Single = CType((accelboundsRectangle.Width / 2.0), Single)
        Dim yCenter As Single = CType((accelboundsRectangle.Height / 2.0), Single)

        accelxyAxisPen = New Pen(Color.Black, 2)
        accelcirclePen = New Pen(Color.DarkBlue, 2)

        accelcircleDiameter = CType((accelDrawingSurface.Width / 1.6), Single)
        accelcircleRadius = accelcircleDiameter / 2

        accelcircleRectangle = New Rectangle(
                                        CType(Math.Round(xCenter - accelcircleRadius), Integer),
                                        CType(Math.Round(yCenter - accelcircleRadius), Integer),
                                        CType(Math.Round(accelcircleDiameter), Integer),
                                        CType(Math.Round(accelcircleDiameter), Integer))
    End Sub

    Private Sub drawAccelGraph(accelData As Double())
        accelGraphicsContext = New BufferedGraphicsContext()
        accelGraphicsBuffer = accelGraphicsContext.Allocate(Graphics.FromImage(accelDrawingSurface), accelboundsRectangle)
        accelGraphicsBuffer.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias
        accelGraphicsBuffer.Graphics.Clear(SystemColors.Control)

        Dim xCenter As Single = CType((accelboundsRectangle.Width / 2.0), Single)
        Dim yCenter As Single = CType((accelboundsRectangle.Height / 2.0), Single)

        Dim xOld As Single = xCenter - CType(accelData(0), Single) * accelcircleRadius * CType((1 / ambientGravity), Single)
        Dim yOld As Single = yCenter - CType(accelData(1), Single) * accelcircleRadius * CType((1 / ambientGravity), Single)

        accelGraphicsBuffer.Graphics.DrawLine(accelxyAxisPen, xCenter, yCenter, xOld, yOld)
        accelGraphicsBuffer.Graphics.DrawEllipse(accelcirclePen, New Rectangle(CType((xCenter - accelcircleRadius), Integer),
                                                                               CType((yCenter - accelcircleRadius), Integer),
                                                                               CType(accelcircleDiameter, Integer),
                                                                               CType(accelcircleDiameter, Integer)))

        Dim zOut As Double = accelData(2) * CType((1 / ambientGravity), Single)
        If zOut > 0 Then
            accelGraphicsBuffer.Graphics.DrawEllipse(New Pen(Color.Red, 2),
                                                     New Rectangle(CType(xCenter, Integer) - CType((accelcircleRadius * zOut), Integer),
                                                                   CType(yCenter, Integer) - CType((accelcircleRadius * zOut), Integer),
                                                                   CType((accelcircleDiameter * zOut), Integer),
                                                                   CType((accelcircleDiameter * zOut), Integer)))
        Else
            accelGraphicsBuffer.Graphics.DrawEllipse(New Pen(Color.Green, 2),
                                                     New Rectangle(CType(xCenter, Integer) - CType((accelcircleRadius * -zOut), Integer),
                                                                   CType(yCenter, Integer) - CType((accelcircleRadius * -zOut), Integer),
                                                                   CType((accelcircleDiameter * -zOut), Integer),
                                                                   CType((accelcircleDiameter * -zOut), Integer)))
        End If

        accelGraphicsBuffer.Render(panel1.CreateGraphics())

        accelGraphicsBuffer.Dispose()

        accelGraphicsContext.Dispose()
    End Sub

    'magnetic field graph
    Dim magFieldGraphicsContext As BufferedGraphicsContext
    Dim magFieldGraphicsBuffer As BufferedGraphics
    Dim magFieldDrawingSurface As Bitmap

    Dim magFieldCircleRectangle, magFieldBoundsRectangle As Rectangle
    Dim magFieldXYAxisPen, magFieldCirclePen As Pen
    Dim magFieldCircleDiameter, magFieldCircleRadius As Single

    Private Sub initMagFieldGraph()
        'initialize our graphical representation of the compasseration data
        magFieldDrawingSurface = New Bitmap(magFieldView.ClientRectangle.Width, magFieldView.ClientRectangle.Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb)

        magFieldBoundsRectangle = New Rectangle(0, 0, magFieldDrawingSurface.Width, magFieldDrawingSurface.Height)
        magFieldBoundsRectangle.Inflate(-1, -1)

        Dim xCenter As Single = CType((magFieldBoundsRectangle.Width / 2.0), Single)
        Dim yCenter As Single = CType((magFieldBoundsRectangle.Height / 2.0), Single)

        magFieldXYAxisPen = New Pen(Color.Black, 2)
        magFieldCirclePen = New Pen(Color.DarkBlue, 2)

        magFieldCircleDiameter = CType((magFieldDrawingSurface.Width / 1.6), Single)
        magFieldCircleRadius = magFieldCircleDiameter / 2

        magFieldCircleRectangle = New Rectangle(CType(Math.Round(xCenter - magFieldCircleRadius), Integer),
                                                CType(Math.Round(yCenter - magFieldCircleRadius), Integer),
                                                CType(Math.Round(magFieldCircleDiameter), Integer),
                                                CType(Math.Round(magFieldCircleDiameter), Integer))
    End Sub

    Private Sub drawMagFieldGraph(compassData As Double())
        magFieldGraphicsContext = New BufferedGraphicsContext()
        magFieldGraphicsBuffer = magFieldGraphicsContext.Allocate(Graphics.FromImage(magFieldDrawingSurface), magFieldBoundsRectangle)
        magFieldGraphicsBuffer.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias
        magFieldGraphicsBuffer.Graphics.Clear(SystemColors.Control)

        Dim xCenter As Single = CType((magFieldBoundsRectangle.Width / 2.0), Single)
        Dim yCenter As Single = CType((magFieldBoundsRectangle.Height / 2.0), Single)

        Dim xOld As Single = xCenter + CType((compassData(0)), Single) * magFieldCircleRadius * CType((1 / ambientMagneticField), Single)
        Dim yOld As Single = yCenter + CType((compassData(1)), Single) * magFieldCircleRadius * CType((1 / ambientMagneticField), Single)

        magFieldGraphicsBuffer.Graphics.DrawLine(magFieldXYAxisPen, xCenter, yCenter, xOld, yOld)
        magFieldGraphicsBuffer.Graphics.DrawEllipse(magFieldCirclePen, magFieldCircleRectangle)

        Dim zOut As Double = (compassData(2)) * CType((1 / ambientMagneticField), Single)
        If zOut > 0 Then
            magFieldGraphicsBuffer.Graphics.DrawEllipse(New Pen(Color.Red, 2),
                                                        New Rectangle(CType(xCenter, Integer) - CType((magFieldCircleRadius * zOut), Integer),
                                                                      CType(yCenter, Integer) - CType((magFieldCircleRadius * zOut), Integer),
                                                                      CType((magFieldCircleDiameter * zOut), Integer),
                                                                      CType((magFieldCircleDiameter * zOut), Integer)))
        Else
            magFieldGraphicsBuffer.Graphics.DrawEllipse(New Pen(Color.Green, 2),
                                                        New Rectangle(CType(xCenter, Integer) - CType((magFieldCircleRadius * -zOut), Integer),
                                                                      CType(yCenter, Integer) - CType((magFieldCircleRadius * -zOut), Integer),
                                                                      CType((magFieldCircleDiameter * -zOut), Integer),
                                                                      CType((magFieldCircleDiameter * -zOut), Integer)))
        End If

        magFieldGraphicsBuffer.Render(magFieldView.CreateGraphics())

        magFieldGraphicsBuffer.Dispose()

        magFieldGraphicsContext.Dispose()
    End Sub

    'compass graph
    Dim compassGraphicsContext As BufferedGraphicsContext
    Dim compassGraphicsBuffer As BufferedGraphics
    Dim compassDrawingSurface As Bitmap

    Dim compassCircleRectangle As RectangleF
    Dim compassboundsRectangle As Rectangle
    Dim compassDotPen, compassCirclePen, compassTickPen, compassTickPenBig As Pen
    Dim compassCircleDiameter, compassCircleRadius As Single


    Private Sub initCompassBearingGraph()
        'initialize our graphical representation of the compasseration data
        compassDrawingSurface = New Bitmap(compassView.ClientRectangle.Width, compassView.ClientRectangle.Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb)

        compassboundsRectangle = New Rectangle(0, 0, compassDrawingSurface.Width, compassDrawingSurface.Height)
        compassboundsRectangle.Inflate(-1, -1)

        Dim xCenter As Single = CType(compassboundsRectangle.Width / 2, Single)
        Dim yCenter As Single = CType(compassboundsRectangle.Height / 2, Single)

        compassDotPen = New Pen(Color.Red, 4)
        compassCirclePen = New Pen(Color.Black, 2)
        compassTickPen = New Pen(Color.DarkBlue, 1)
        compassTickPenBig = New Pen(Color.DarkBlue, 2)

        compassCircleDiameter = CType(compassDrawingSurface.Width / 1.35, Single)
        compassCircleRadius = CType(compassCircleDiameter / 2.0, Single)

        compassCircleRectangle = New RectangleF(xCenter - compassCircleRadius, yCenter - compassCircleRadius, compassCircleDiameter, compassCircleDiameter)
    End Sub

    Private Sub drawCompassBearingGraph()
        compassGraphicsContext = New BufferedGraphicsContext()
        compassGraphicsBuffer = compassGraphicsContext.Allocate(Graphics.FromImage(compassDrawingSurface), compassboundsRectangle)
        compassGraphicsBuffer.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias
        compassGraphicsBuffer.Graphics.Clear(SystemColors.Control)

        Dim xCenter As Single = CType(compassboundsRectangle.Width / 2, Single)
        Dim yCenter As Single = CType(compassboundsRectangle.Height / 2, Single)

        compassGraphicsBuffer.Graphics.DrawEllipse(compassCirclePen, compassCircleRectangle)

        Dim pt As Point = New Point(CType(xCenter - spatialImage.Width / 2, Integer), CType(yCenter - spatialImage.Height / 2, Integer))
        compassGraphicsBuffer.Graphics.DrawImageUnscaled(rotateImage(spatialImage, CType(compassBearing, Single)), pt)

        compassGraphicsBuffer.Graphics.DrawString("N", New Font("Arial", 16), New SolidBrush(Color.Black), xCenter - 10, 0)
        compassGraphicsBuffer.Graphics.DrawString("E", New Font("Arial", 16), New SolidBrush(Color.Black), xCenter * 2 - 20, yCenter - 12)
        compassGraphicsBuffer.Graphics.DrawString("S", New Font("Arial", 16), New SolidBrush(Color.Black), xCenter - 10, yCenter * 2 - 20)
        compassGraphicsBuffer.Graphics.DrawString("W", New Font("Arial", 16), New SolidBrush(Color.Black), 0, yCenter - 12)

        'ticks around the compass circle
        For i As Integer = 0 To 359 Step 10
            Dim p As Pen = compassTickPen
            Dim tickSize As Integer = 4
            If i = 0 Or i = 90 Or i = 180 Or i = 270 Then
                p = compassTickPenBig
                tickSize = 6
            End If
            compassGraphicsBuffer.Graphics.DrawLine(p,
                                                   CType((compassCircleRadius + tickSize) * Math.Cos(i * (Math.PI / 180)), Single) + xCenter,
                                                   CType(-(compassCircleRadius + tickSize) * Math.Sin(i * (Math.PI / 180)), Single) + yCenter,
                                                   CType((compassCircleRadius - tickSize) * Math.Cos(i * (Math.PI / 180)), Single) + xCenter,
                                                   CType(-(compassCircleRadius - tickSize) * Math.Sin(i * (Math.PI / 180)), Single) + yCenter)
        Next

        'marker on the compass circle
        compassGraphicsBuffer.Graphics.DrawEllipse(compassDotPen,
                                                   CType(compassCircleRadius * Math.Cos((-compassBearing + 90) * Math.PI / 180) + xCenter, Single) - 2,
                                                   CType(-compassCircleRadius * Math.Sin((-compassBearing + 90) * Math.PI / 180) + yCenter, Single) - 2,
                                                   4,
                                                   4)

        compassGraphicsBuffer.Render(compassView.CreateGraphics())
        compassGraphicsBuffer.Dispose()
        compassGraphicsContext.Dispose()
    End Sub

    Private Function rotateImage(ByVal b As Bitmap, ByVal angle As Single) As Bitmap
        'create a new empty bitmap to hold rotated image
        Dim returnBitmap As Bitmap = New Bitmap(b.Width, b.Height)
        'make a graphics object from the empty bitmap
        Dim g As Graphics = Graphics.FromImage(returnBitmap)
        'move rotation point to center of image
        g.TranslateTransform(CType(b.Width / 2, Single), CType(b.Height / 2, Single))
        'rotate
        g.RotateTransform(angle)
        'move image back
        g.TranslateTransform(-CType(b.Width / 2, Single), -CType(b.Height / 2, Single))
        'draw passed in image onto craphics object
        g.DrawImage(b, New Point(0, 0))
        Return returnBitmap
    End Function
#End Region

    Private Sub setCompassParamsButton_Click(sender As Object, e As EventArgs) Handles setCompassParamsButton.Click
        Dim calibrationString As String = "0.41595, 0.02233, 0.15236, -0.22662, 2.36152, 2.27444, 2.57652, -0.03768, -0.01708, -0.03607, -0.00707, -0.01822, -0.00787" 'test string based on Calgary, AB and a prototype magnetometer

        Dim tib As ParameterInputBox = New ParameterInputBox(calibrationString)
        Dim result As DialogResult
        result = tib.ShowDialog()

        If result = DialogResult.OK Then
            If tib.parameters.Length > 0 Then
                calibrationString = tib.parameters
            Else
                calibrationStatusTxt.Text = "Un-Calibrated"
                calibrationStatusTxt.ForeColor = Color.Red
                calibrationStatusTxt.Visible = True
            End If

            Dim paramVals() As String = calibrationString.Split(New Char() {","})
            If paramVals.Length = 13 Then
                Try
                    device.SetMagnetometerCorrectionParameters(
                        Double.Parse(paramVals(0)),
                        Double.Parse(paramVals(1)),
                        Double.Parse(paramVals(2)),
                        Double.Parse(paramVals(3)),
                        Double.Parse(paramVals(4)),
                        Double.Parse(paramVals(5)),
                        Double.Parse(paramVals(6)),
                        Double.Parse(paramVals(7)),
                        Double.Parse(paramVals(8)),
                        Double.Parse(paramVals(9)),
                        Double.Parse(paramVals(10)),
                        Double.Parse(paramVals(11)),
                        Double.Parse(paramVals(12)))
                    ambientMagneticField = Double.Parse(paramVals(0))
                    calibrationStatusTxt.Text = "Calibrated"
                    calibrationStatusTxt.ForeColor = Color.Green
                    calibrationStatusTxt.Visible = True
                Catch ex As PhidgetException
                    errorBox.addMessage("Error calibrating: " + ex.Message)
                End Try
            End If
        End If
    End Sub

    Private Sub zeroGyroButton_Click(sender As Object, e As EventArgs) Handles zeroGyroButton.Click
        device.ZeroGyroscope()
        Threading.Thread.Sleep(100)
        gyroHeading(0) = 0
        gyroHeading(1) = 0
        gyroHeading(2) = 0
    End Sub

End Class
